package org.netkernel.layer1.endpoint;

import java.util.concurrent.ConcurrentHashMap;
import java.util.*;
import javax.xml.parsers.ParserConfigurationException;

import org.netkernel.container.IKernel;
import org.netkernel.container.ISpaceListener;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.IStandardEndpoint;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.ISpace;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class JavaRuntimeAccessor extends StandardAccessorImpl implements ISpaceListener
{
	private Element mGrammar;
	private Map<String, CachedEndpoint> mEndpointCache = new ConcurrentHashMap<String, CachedEndpoint>();	
	
	public JavaRuntimeAccessor() throws ParserConfigurationException
	{
		declareThreadSafe();
		
		Document d=XMLUtils.newDocument();
		Element g=d.createElement("grammar");
		d.appendChild(g);
		Element a=d.createElement("active");
		g.appendChild(a);
		Element t=d.createElement("identifier");
		XMLUtils.setText(t, "active:java");
		a.appendChild(t);
		Element v=d.createElement("varargs");
		a.appendChild(v);
		mGrammar=g;
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	super.onCommissionEndpoint(aKernel, aSpace);
		aKernel.addSpaceListener(this);
	}
	
	public void onDecommissionEndpoint() throws Exception
	{	getKernel().removeSpaceListener(this);
		super.onDecommissionEndpoint();
	}	
	
	public void spaceChanged()
	{	mEndpointCache.clear();
	}	
	
	public void onRequest(INKFRequestContext aContext) throws Exception
	{
		if (aContext.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
		{	super.onMeta(aContext);
		}
		else
		{	String className = aContext.getThisRequest().getArgumentValue("class");
			IStandardEndpoint ep;
			
			CachedEndpoint ce = mEndpointCache.get(className);
			if (ce!=null)
			{	ep=ce.getEndpoint();
			}
			else
			{	RequestScopeClassLoader cl = new RequestScopeClassLoader(aContext.getKernelContext().getThisKernelRequest().getRequestScope());
				Class c = cl.loadClass(className);
				try
				{	ep = (IStandardEndpoint)c.newInstance();
				}
				catch (ClassCastException e)
				{	throw aContext.createFormattedException("EX_JAVA_NOT_ACCESSOR", "MSG_JAVA_NOT_ACCESSOR", null, null, c.getName());
				}
				ep.setParameterRaw("grammar", mGrammar, getKernel());
				ep.onCommissionEndpoint(aContext.getKernelContext().getKernel(), this.getSpace());
				
				ce = new CachedEndpoint(ep);
				mEndpointCache.put(className, ce);
			}
					
			aContext.setNoResponse(); //endpoint will do the response
			ep.onAsyncRequest(aContext.getKernelContext().getThisKernelRequest());
		}
	}
	
	/** Holder class around endpoint instance in case we want to hold LRU etc in future
	 * @author tab
	 *
	 */
	private static class CachedEndpoint
	{	private IStandardEndpoint mEndpoint;
	
		public CachedEndpoint(IStandardEndpoint aEndpoint)
		{	mEndpoint=aEndpoint;
		}
		
		public IStandardEndpoint getEndpoint()
		{	return mEndpoint;
		}
	}
}