package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFEndpointImpl;
import org.netkernel.layer0.nkf.impl.NKFSpaceImpl;
import org.netkernel.module.standard.endpoint.TransparentOverlayImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.IEndpoint;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class ScratchPadOverlay extends TransparentOverlayImpl
{
	public ScratchPadOverlay()
	{	declareThreadSafe();
		declareUnhandledExceptionsExpired(false);
	}
	public void onRequest(String aID, INKFRequestContext aContext) throws Exception
	{	
		INKFRequest reqOut=aContext.getThisRequest().getIssuableClone();
		IRequestScopeLevel scope=aContext.getKernelContext().getThisKernelRequest().getRequestScope();
		ScratchPadSpace space=new ScratchPadSpace();
		space.onCommissionSpace(aContext.getKernelContext().getKernel());
		scope=RequestScopeLevelImpl.createDurableScopeLevel(space, scope);
		scope=RequestScopeLevelImpl.createNonDurableScopeLevel(getDelegateSpace(), scope);
		reqOut.setRequestScope(scope);
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(reqOut);
		aContext.createResponseFrom(resp);	
	}
	
	
	private static class ScratchPadSpace extends NKFSpaceImpl
	{
		private static IEndpoint sEndpoint = new ScratchPadEndpoint();
		private static final String SCHEME="scratch:";
		private Map<String,Object> mValueMap = Collections.synchronizedMap(new HashMap<String,Object>());
		
		public void onResolve(INKFResolutionContext helper) throws Exception
		{	String requestId = helper.getRequestToResolve().getIdentifier();
			if(requestId.startsWith(SCHEME))
			{
				sEndpoint.onCommissionEndpoint(getKernel(), this);
				helper.createResolutionResponse(sEndpoint);
			}
		}

		public void handleRequestFromEndpoint(INKFRequestContext helper) throws Exception
		{	String identifier=helper.getThisRequest().getIdentifier();
			Object response=null;
			switch(helper.getThisRequest().getVerb())
			{	case INKFRequestReadOnly.VERB_SOURCE:
					response=mValueMap.get(identifier);
					if (response==null)
					{	throw new Exception(identifier+" not found");
					}
				break;
				case INKFRequestReadOnly.VERB_SINK:
                    Object primary = helper.getThisRequest().getPrimary();
					mValueMap.put(identifier,primary);
				break;
				case INKFRequestReadOnly.VERB_DELETE:
					Object removed=mValueMap.remove(identifier);
					response=removed!=null;
				break;
				case INKFRequestReadOnly.VERB_EXISTS:
					response= mValueMap.containsKey(identifier);
				break;
			}
			INKFResponse resp=helper.createResponseFrom(response);			
		}
		
		public String toString()
		{	return "ScratchPadSpace";
		}
	}
	
	private static class ScratchPadEndpoint extends NKFEndpointImpl
	{
		public ScratchPadEndpoint()
		{	super(true);
		}
		
		public void onRequest(INKFRequestContext aContext) throws Exception
		{	ScratchPadSpace space=(ScratchPadSpace)aContext.getKernelContext().getThisKernelRequest().getRequestScope().getSpace();
			space.handleRequestFromEndpoint(aContext);
		}
	}
}
