/******************************************************************************
  (c) Copyright 2002-2006, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile$
  Version:       $Name$ $Revision$
  Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

import java.io.*;

public class ToString extends StandardTransreptorImpl
{
    private static final int DEFAULT_CONVERSION_BUFFER_SIZE_BYTES = 1024;
	public ToString()
	{	this.declareThreadSafe();
		this.declareFromRepresentation(IBinaryStreamRepresentation.class);
		this.declareToRepresentation(String.class);
	}
	
	public void onTransrept(INKFRequestContext context) throws Exception
	{
		INKFResponseReadOnly fromResp=context.getThisRequest().getPrimaryAsResponse();
		Object from=fromResp.getRepresentation();
		String encoding;
		String result=null;
		if (from instanceof IReadableBinaryStreamRepresentation)
		{	// create from readable binary stream
			IReadableBinaryStreamRepresentation bs = (IReadableBinaryStreamRepresentation)from;
			int length = bs.getContentLength();
			if (0 < length)
			{
				encoding = bs.getEncoding();
				if (encoding==null) encoding="UTF-8";
				InputStream is=bs.getInputStream();
				try
				{	InputStreamReader br = new InputStreamReader(is,encoding);
					char[] buffer = new char[length];
					int l=br.read(buffer,0, length);
					result = new String(buffer,0,l);
				}
				finally
				{	is.close();
				}
			}
		}
		else if (from instanceof IBinaryStreamRepresentation)
		{	// create from regular binary stream
			IBinaryStreamRepresentation bs = (IBinaryStreamRepresentation)from;
			ByteArrayOutputStream baos = new ByteArrayOutputStream(DEFAULT_CONVERSION_BUFFER_SIZE_BYTES);
			bs.write(baos);
			encoding = bs.getEncoding();
			if (encoding==null) encoding="UTF-8";
			result = baos.toString(encoding);		
		}		
		else
		{	result = from.toString();
		}
		
		
		INKFResponse response = context.createResponseFrom( result );
		response.setMimeType(fromResp.getMimeType());
	}
}